# flags: --extend-ignore=Y023

import abc
import builtins
import collections.abc
import typing
from abc import abstractmethod
from collections.abc import AsyncIterable, AsyncIterator, Iterable, Iterator
from typing import Any, overload

import typing_extensions
from _typeshed import Self
from typing_extensions import final


class Bad(
    object
):  # Y040 Do not inherit from "object" explicitly, as it is redundant in Python 3
    def __new__(cls, *args: Any, **kwargs: Any) -> Bad:
        ...  # Y034 "__new__" methods usually return "self" at runtime. Consider using "typing_extensions.Self" in "Bad.__new__", e.g. "def __new__(cls, *args: Any, **kwargs: Any) -> Self: ..."

    def __repr__(self) -> str:
        ...  # Y029 Defining __repr__ or __str__ in a stub is almost always redundant

    def __str__(self) -> builtins.str:
        ...  # Y029 Defining __repr__ or __str__ in a stub is almost always redundant

    def __eq__(self, other: Any) -> bool:
        ...  # Y032 Prefer "object" to "Any" for the second parameter in "__eq__" methods

    def __ne__(self, other: typing.Any) -> typing.Any:
        ...  # Y032 Prefer "object" to "Any" for the second parameter in "__ne__" methods

    def __enter__(self) -> Bad:
        ...  # Y034 "__enter__" methods in classes like "Bad" usually return "self" at runtime. Consider using "typing_extensions.Self" in "Bad.__enter__", e.g. "def __enter__(self) -> Self: ..."

    async def __aenter__(self) -> Bad:
        ...  # Y034 "__aenter__" methods in classes like "Bad" usually return "self" at runtime. Consider using "typing_extensions.Self" in "Bad.__aenter__", e.g. "async def __aenter__(self) -> Self: ..."

    def __iadd__(self, other: Bad) -> Bad:
        ...  # Y034 "__iadd__" methods in classes like "Bad" usually return "self" at runtime. Consider using "typing_extensions.Self" in "Bad.__iadd__", e.g. "def __iadd__(self, other: Bad) -> Self: ..."


class AlsoBad(int, builtins.object):
    ...  # Y040 Do not inherit from "object" explicitly, as it is redundant in Python 3


class Good:
    def __new__(cls: type[Self], *args: Any, **kwargs: Any) -> Self:
        ...

    @abstractmethod
    def __str__(self) -> str:
        ...

    @abc.abstractmethod
    def __repr__(self) -> str:
        ...

    def __eq__(self, other: object) -> bool:
        ...

    def __ne__(self, obj: object) -> int:
        ...

    def __enter__(self: Self) -> Self:
        ...

    async def __aenter__(self: Self) -> Self:
        ...

    def __ior__(self: Self, other: Self) -> Self:
        ...


class Fine:
    @overload
    def __new__(cls, foo: int) -> FineSubclass:
        ...

    @overload
    def __new__(cls, *args: Any, **kwargs: Any) -> Fine:
        ...

    @abc.abstractmethod
    def __str__(self) -> str:
        ...

    @abc.abstractmethod
    def __repr__(self) -> str:
        ...

    def __eq__(self, other: Any, strange_extra_arg: list[str]) -> Any:
        ...

    def __ne__(self, *, kw_only_other: Any) -> bool:
        ...

    def __enter__(self) -> None:
        ...

    async def __aenter__(self) -> bool:
        ...


class FineSubclass(Fine):
    ...


class StrangeButAcceptable(str):
    @typing_extensions.overload
    def __new__(cls, foo: int) -> StrangeButAcceptableSubclass:
        ...

    @typing_extensions.overload
    def __new__(cls, *args: Any, **kwargs: Any) -> StrangeButAcceptable:
        ...

    def __str__(self) -> StrangeButAcceptable:
        ...

    def __repr__(self) -> StrangeButAcceptable:
        ...


class StrangeButAcceptableSubclass(StrangeButAcceptable):
    ...


class FineAndDandy:
    def __str__(self, weird_extra_arg) -> str:
        ...

    def __repr__(self, weird_extra_arg_with_default=...) -> str:
        ...


@final
class WillNotBeSubclassed:
    def __new__(cls, *args: Any, **kwargs: Any) -> WillNotBeSubclassed:
        ...

    def __enter__(self) -> WillNotBeSubclassed:
        ...

    async def __aenter__(self) -> WillNotBeSubclassed:
        ...


# we don't emit an error for these; out of scope for a linter
class InvalidButPluginDoesNotCrash:
    def __new__() -> InvalidButPluginDoesNotCrash:
        ...

    def __enter__() -> InvalidButPluginDoesNotCrash:
        ...

    async def __aenter__() -> InvalidButPluginDoesNotCrash:
        ...


class BadIterator1(Iterator[int]):
    def __iter__(self) -> Iterator[int]:
        ...  # Y034 "__iter__" methods in classes like "BadIterator1" usually return "self" at runtime. Consider using "typing_extensions.Self" in "BadIterator1.__iter__", e.g. "def __iter__(self) -> Self: ..."


class BadIterator2(
    typing.Iterator[int]
):  # Y022 Use "collections.abc.Iterator[T]" instead of "typing.Iterator[T]" (PEP 585 syntax)
    def __iter__(self) -> Iterator[int]:
        ...  # Y034 "__iter__" methods in classes like "BadIterator2" usually return "self" at runtime. Consider using "typing_extensions.Self" in "BadIterator2.__iter__", e.g. "def __iter__(self) -> Self: ..."


class BadIterator3(
    typing.Iterator[int]
):  # Y022 Use "collections.abc.Iterator[T]" instead of "typing.Iterator[T]" (PEP 585 syntax)
    def __iter__(self) -> collections.abc.Iterator[int]:
        ...  # Y034 "__iter__" methods in classes like "BadIterator3" usually return "self" at runtime. Consider using "typing_extensions.Self" in "BadIterator3.__iter__", e.g. "def __iter__(self) -> Self: ..."


class BadIterator4(Iterator[int]):
    # Note: *Iterable*, not *Iterator*, returned!
    def __iter__(self) -> Iterable[int]:
        ...  # Y034 "__iter__" methods in classes like "BadIterator4" usually return "self" at runtime. Consider using "typing_extensions.Self" in "BadIterator4.__iter__", e.g. "def __iter__(self) -> Self: ..."


class IteratorReturningIterable:
    def __iter__(self) -> Iterable[str]:
        ...  # Y045 "__iter__" methods should return an Iterator, not an Iterable


class BadAsyncIterator(collections.abc.AsyncIterator[str]):
    def __aiter__(self) -> typing.AsyncIterator[str]:
        ...  # Y034 "__aiter__" methods in classes like "BadAsyncIterator" usually return "self" at runtime. Consider using "typing_extensions.Self" in "BadAsyncIterator.__aiter__", e.g. "def __aiter__(self) -> Self: ..."  # Y022 Use "collections.abc.AsyncIterator[T]" instead of "typing.AsyncIterator[T]" (PEP 585 syntax)


class AsyncIteratorReturningAsyncIterable:
    def __aiter__(self) -> AsyncIterable[str]:
        ...  # Y045 "__aiter__" methods should return an AsyncIterator, not an AsyncIterable


class Abstract(Iterator[str]):
    @abstractmethod
    def __iter__(self) -> Iterator[str]:
        ...

    @abstractmethod
    def __enter__(self) -> Abstract:
        ...

    @abstractmethod
    async def __aenter__(self) -> Abstract:
        ...


class GoodIterator(Iterator[str]):
    def __iter__(self: Self) -> Self:
        ...


class GoodAsyncIterator(AsyncIterator[int]):
    def __aiter__(self: Self) -> Self:
        ...


class DoesNotInheritFromIterator:
    def __iter__(self) -> DoesNotInheritFromIterator:
        ...


class Unannotated:
    def __new__(cls, *args, **kwargs):
        ...

    def __iter__(self):
        ...

    def __aiter__(self):
        ...

    async def __aenter__(self):
        ...

    def __repr__(self):
        ...

    def __str__(self):
        ...

    def __eq__(self):
        ...

    def __ne__(self):
        ...

    def __iadd__(self):
        ...

    def __ior__(self):
        ...


def __repr__(self) -> str:
    ...


def __str__(self) -> str:
    ...


def __eq__(self, other: Any) -> bool:
    ...


def __ne__(self, other: Any) -> bool:
    ...


def __imul__(self, other: Any) -> list[str]:
    ...



min(1, 2, 3)
min(1, min(2, 3))
min(1, min(2, min(3, 4)))
min(1, foo("a", "b"), min(3, 4))
min(1, max(2, 3))
max(1, 2, 3)
max(1, max(2, 3))
max(1, max(2, max(3, 4)))
max(1, foo("a", "b"), max(3, 4))

# These should not trigger; we do not flag cases with keyword args.
min(1, min(2, 3), key=test)
min(1, min(2, 3, key=test))
# This will still trigger, to merge the calls without keyword args.
min(1, min(2, 3, key=test), min(4, 5))

# Don't provide a fix if there are comments within the call.
min(
    1,  # This is a comment.
    min(2, 3),
)

# Handle iterable expressions.
min(1, min(a))
min(1, min(i for i in range(10)))
max(1, max(a))
max(1, max(i for i in range(10)))

tuples_list = [
    (1, 2),
    (2, 3),
    (3, 4),
    (4, 5),
    (5, 6),
]

min(min(tuples_list))
max(max(tuples_list))

# Starred argument should be copied as it is.
max(1, max(*a))



def f(x, y, z, t, u, v, w, r):  # Too many arguments (8/5)
    pass


def f(x, y, z, t, u):  # OK
    pass


def f(x):  # OK
    pass


def f(x, y, z, _t, _u, _v, _w, r):  # OK (underscore-prefixed names are ignored
    pass


def f(x, y, z, u=1, v=1, r=1):  # Too many arguments (6/5)
    pass


def f(x=1, y=1, z=1):  # OK
    pass


def f(x, y, z, /, u, v, w):  # Too many arguments (6/5)
    pass


def f(x, y, z, *, u, v, w):  # Too many arguments (6/5)
    pass


def f(x, y, z, a, b, c, *, u, v, w):  # Too many arguments (9/5)
    pass



# Errors.
nums = [1, 2, 3]
map(lambda x: x + 1, nums)
map(lambda x: str(x), nums)
list(map(lambda x: x * 2, nums))
set(map(lambda x: x % 2 == 0, nums))
dict(map(lambda v: (v, v**2), nums))
dict(map(lambda v: [v, v**2], nums))
map(lambda: "const", nums)
map(lambda _: 3.0, nums)
_ = "".join(map(lambda x: x in nums and "1" or "0", range(123)))
all(map(lambda v: isinstance(v, dict), nums))
filter(func, map(lambda v: v, nums))
list(map(lambda x, y: x * y, nums))

# When inside f-string, then the fix should be surrounded by whitespace
_ = f"{set(map(lambda x: x % 2 == 0, nums))}"
_ = f"{dict(map(lambda v: (v, v**2), nums))}"

# False negatives.
map(lambda x=2, y=1: x + y, nums, nums)
set(map(lambda x, y: x, nums, nums))


def func(arg1: int, arg2: int = 4):
    return 2 * arg1 + arg2


# Non-error: `func` is not a lambda.
list(map(func, nums))

# False positive: need to preserve the late-binding of `x` in the inner lambda.
map(lambda x: lambda: x, range(4))

# Error: the `x` is overridden by the inner lambda.
map(lambda x: lambda x: x, range(4))

# Ok because of the default parameters, and variadic arguments.
map(lambda x=1: x, nums)
map(lambda *args: len(args), range(4))
map(lambda **kwargs: len(kwargs), range(4))

# Ok because multiple arguments are allowed.
dict(map(lambda k, v: (k, v), keys, values))

# Regression test for: https://github.com/astral-sh/ruff/issues/7121
map(lambda x: x, y if y else z)
map(lambda x: x, (y if y else z))
map(lambda x: x, (x, y, z))



import os
import glob
from glob import glob as search


extensions_dir = "./extensions"

# PTH207
glob.glob(os.path.join(extensions_dir, "ops", "autograd", "*.cpp"))
list(glob.iglob(os.path.join(extensions_dir, "ops", "autograd", "*.cpp")))
search("*.png")



# Standard library.
import os

# First-party.
from airflow.jobs.scheduler_job_runner import SchedulerJobRunner

# Stub file.
from airflow.compat.functools import cached_property

# Namespace package.
from airflow.providers.google.cloud.hooks.gcs import GCSHook

# Third-party.
from sqlalchemy.orm import Query



import typing
from dataclasses import dataclass, field
from typing import ClassVar, Sequence

KNOWINGLY_MUTABLE_DEFAULT = []


@dataclass
class A:
    mutable_default: list[int] = []
    immutable_annotation: typing.Sequence[int] = []
    without_annotation = []
    correct_code: list[int] = KNOWINGLY_MUTABLE_DEFAULT
    perfectly_fine: list[int] = field(default_factory=list)
    class_variable: typing.ClassVar[list[int]] = []


@dataclass
class B:
    mutable_default: list[int] = []
    immutable_annotation: Sequence[int] = []
    without_annotation = []
    correct_code: list[int] = KNOWINGLY_MUTABLE_DEFAULT
    perfectly_fine: list[int] = field(default_factory=list)
    class_variable: ClassVar[list[int]] = []



def f():
    from collections.abc import Set as AbstractSet  # Ok

def f():
    from collections.abc import Container, Sized, Set as AbstractSet, ValuesView  # Ok

def f():
    from collections.abc import Set  # PYI025

def f():
    from collections.abc import Container, Sized, Set, ValuesView  # PYI025

def f():
    """Test: local symbol renaming."""
    if True:
        from collections.abc import Set
    else:
        Set = 1

    x: Set = set()

    x: Set

    del Set

    def f():
        print(Set)

        def Set():
            pass
        print(Set)

from collections.abc import Set

def f():
    """Test: global symbol renaming."""
    global Set

    Set = 1
    print(Set)

def f():
    """Test: nonlocal symbol renaming."""
    from collections.abc import Set

    def g():
        nonlocal Set

        Set = 1
        print(Set)



s = set([x for x in range(3)])
s = set(
    [x for x in range(3)]
)

s = f"{set([x for x in 'ab'])}"
s = f'{set([x for x in "ab"])}'

def f(x):
    return x

s = f"{set([f(x) for x in 'ab'])}"

s = f"{ set([x for x in 'ab']) | set([x for x in 'ab']) }"
s = f"{set([x for x in 'ab']) | set([x for x in 'ab'])}"



import logging
from logging import getLogger

# Ok
logging.getLogger(__name__)
logging.getLogger(name=__name__)
logging.getLogger("custom")
logging.getLogger(name="custom")

# LOG002
getLogger(__file__)
logging.getLogger(name=__file__)

logging.getLogger(__cached__)
getLogger(name=__cached__)


# Override `logging.getLogger`
class logging:
    def getLogger(self):
        pass


logging.getLogger(__file__)



bla = b"bla"
d = {"a": b"bla", "b": b"bla", "c": b"bla"}


def foo(one_arg):
    pass


f"{str(bla)}, {repr(bla)}, {ascii(bla)}"  # RUF010

f"{str(d['a'])}, {repr(d['b'])}, {ascii(d['c'])}"  # RUF010

f"{(str(bla))}, {(repr(bla))}, {(ascii(bla))}"  # RUF010

f"{bla!s}, {(repr(bla))}, {(ascii(bla))}"  # RUF010

f"{foo(bla)}"  # OK

f"{str(bla, 'ascii')}, {str(bla, encoding='cp1255')}"  # OK

f"{bla!s} {[]!r} {'bar'!a}"  # OK

"Not an f-string {str(bla)}, {repr(bla)}, {ascii(bla)}"  # OK


def ascii(arg):
    pass


f"{ascii(bla)}"  # OK

(
    f"Member of tuple mismatches type at index {i}. Expected {of_shape_i}. Got "
    " intermediary content "
    f" that flows {repr(obj)} of type {type(obj)}.{additional_message}"  # RUF010
)


# OK
f"{str({})}"



import abc


class Person:
    def developer_greeting(self, name):  # [no-self-use]
        print(f"Greetings {name}!")

    def greeting_1(self):  # [no-self-use]
        print("Hello!")

    def greeting_2(self):  # [no-self-use]
        print("Hi!")


# OK
def developer_greeting():
    print("Greetings developer!")


# OK
class Person:
    name = "Paris"

    def __init__(self):
        pass

    def __cmp__(self, other):
        print(24)

    def __repr__(self):
        return "Person"

    def func(self):
        ...

    def greeting_1(self):
        print(f"Hello from {self.name} !")

    @staticmethod
    def greeting_2():
        print("Hi!")


class Base(abc.ABC):
    """abstract class"""

    @abstractmethod
    def abstract_method(self):
        """abstract method could not be a function"""
        raise NotImplementedError


class Sub(Base):
    @override
    def abstract_method(self):
        print("concrete method")


class Prop:
    @property
    def count(self):
        return 24



def f():
    from collections.abc import Set as AbstractSet  # Ok


def f():
    from collections.abc import Container, Sized, Set as AbstractSet, ValuesView  # Ok


def f():
    from collections.abc import Set  # PYI025


def f():
    from collections.abc import Container, Sized, Set, ValuesView  # PYI025

    GLOBAL: Set[int] = set()

    class Class:
        member: Set[int]



"""Test type parameters and aliases"""

# Type parameters in type alias statements

from some_module import Bar

type Foo[T] = T # OK
type Foo[T] = list[T] # OK
type Foo[T: ForwardA] = T # OK
type Foo[*Ts] = Bar[Ts] # OK
type Foo[**P] = Bar[P] # OK
class ForwardA: ...

# Types used in aliased assignment must exist

type Foo = DoesNotExist  # F821: Undefined name `DoesNotExist`
type Foo = list[DoesNotExist]  # F821: Undefined name `DoesNotExist`

# Type parameters do not escape alias scopes

type Foo[T] = T
T  # F821: Undefined name `T` - not accessible afterward alias scope

# Type parameters in functions

def foo[T](t: T) -> T: return t  # OK
async def afoo[T](t: T) -> T: return t  # OK
def with_forward_ref[T: ForwardB](t: T) -> T: return t  # OK
def can_access_inside[T](t: T) -> T:  # OK
    print(T)  # OK
    return t  # OK
class ForwardB: ...


# Type parameters do not escape function scopes

from some_library import some_decorator

@some_decorator(T)  # F821: Undefined name `T` - not accessible in decorators

def foo[T](t: T) -> None: ...
T  # F821: Undefined name `T` - not accessible afterward function scope


# Type parameters in classes

class Foo[T](list[T]): ... # OK
class UsesForward[T: ForwardC](list[T]): ...    # OK
class ForwardC: ...
class WithinBody[T](list[T]):   # OK
    t = T   # OK
    x: T  # OK

    def foo(self, x: T) -> T:  # OK
        return x
    
    def foo(self):
        T  # OK


# Type parameters do not escape class scopes

from some_library import some_decorator
@some_decorator(T)  # F821: Undefined name `T` - not accessible in decorators

class Foo[T](list[T]): ...
T  # F821: Undefined name `T` - not accessible after class scope

# Types specified in bounds should exist

type Foo[T: DoesNotExist] = T  # F821: Undefined name `DoesNotExist`
def foo[T: DoesNotExist](t: T) -> T: return t  # F821: Undefined name `DoesNotExist`
class Foo[T: DoesNotExist](list[T]): ...  # F821: Undefined name `DoesNotExist`

type Foo[T: (DoesNotExist1, DoesNotExist2)] = T  # F821: Undefined name `DoesNotExist1`, Undefined name `DoesNotExist2`
def foo[T: (DoesNotExist1, DoesNotExist2)](t: T) -> T: return t  # F821: Undefined name `DoesNotExist1`, Undefined name `DoesNotExist2`
class Foo[T: (DoesNotExist1, DoesNotExist2)](list[T]): ...  # F821: Undefined name `DoesNotExist1`, Undefined name `DoesNotExist2`

# Type parameters in nested classes

class Parent[T]:
    t = T   # OK

    def can_use_class_variable(self, x: t) -> t:  # OK
        return x
    
    class Child:
        def can_access_parent_type_parameter(self, x: T) -> T:  # OK
            T  # OK
            return x
        
        def cannot_access_parent_variable(self, x: t) -> t:  # F821: Undefined name `T`
                t # F821: Undefined name `t`
                return x
            
# Type parameters in nested functions

def can_access_inside_nested[T](t: T) -> T:  # OK
    def bar(x: T) -> T:  # OK
        T # OK
        return x
    
    bar(t)



def func():
    if False:
        return 0
    return 1

def func():
    if True:
        return 1
    return 0

def func():
    if False:
        return 0
    else:
        return 1

def func():
    if True:
        return 1
    else:
        return 0

def func():
    if False:
        return 0
    else:
        return 1
    return "unreachable"

def func():
    if True:
        return 1
    else:
        return 0
    return "unreachable"

def func():
    if True:
        if True:
            return 1
        return 2
    else:
        return 3
    return "unreachable2"

def func():
    if False:
        return 0

def func():
    if True:
        return 1

def func():
    if True:
        return 1
    elif False:
        return 2
    else:
        return 0

def func():
    if False:
        return 1
    elif True:
        return 2
    else:
        return 0

def func():
    if True:
        if False:
            return 0
        elif True:
            return 1
        else:
            return 2
        return 3
    elif True:
        return 4
    else:
        return 5
    return 6

def func():
    if False:
        return "unreached"
    elif False:
        return "also unreached"
    return "reached"

# Test case found in the Bokeh repository that trigger a false positive.
def func(self, obj: BytesRep) -> bytes:
    data = obj["data"]

    if isinstance(data, str):
        return base64.b64decode(data)
    elif isinstance(data, Buffer):
        buffer = data
    else:
        id = data["id"]

        if id in self._buffers:
            buffer = self._buffers[id]
        else:
            self.error(f"can't resolve buffer '{id}'")

    return buffer.data



import custom
from custom import ImmutableTypeB


def okay(foo: ImmutableTypeB = []):
    ...


def okay(foo: custom.ImmutableTypeA = []):
    ...


def okay(foo: custom.ImmutableTypeB = []):
    ...


def error_due_to_missing_import(foo: ImmutableTypeA = []):
    ...



"""Test case expected to be run with `mypy_init_return = True`."""

# Error
class Foo:
    def __init__(self):
        ...


# Error
class Foo:
    def __init__(self, foo):
        ...


# OK
class Foo:
    def __init__(self, foo) -> None:
        ...


# OK
class Foo:
    def __init__(self) -> None:
        ...


# OK
class Foo:
    def __init__(self, foo: int):
        ...


# OK
class Foo:
    def __init__(self, foo: int) -> None:
        ...


# Error
def __init__(self, foo: int):
    ...


# Error – used to be ok for a moment since the mere presence
# of a vararg falsely indicated that the function has a typed argument.
class Foo:
    def __init__(self, *arg):
        ...



import collections.abc
import typing
from collections.abc import Iterator, Iterable


class NoReturn:
    def __iter__(self):
        ...


class TypingIterableTReturn:
    def __iter__(self) -> typing.Iterable[int]:
        ...

    def not_iter(self) -> typing.Iterable[int]:
        ...


class TypingIterableReturn:
    def __iter__(self) -> typing.Iterable:
        ...

    def not_iter(self) -> typing.Iterable:
        ...


class CollectionsIterableTReturn:
    def __iter__(self) -> collections.abc.Iterable[int]:
        ...

    def not_iter(self) -> collections.abc.Iterable[int]:
        ...


class CollectionsIterableReturn:
    def __iter__(self) -> collections.abc.Iterable:
        ...

    def not_iter(self) -> collections.abc.Iterable:
        ...


class IterableReturn:
    def __iter__(self) -> Iterable:
        ...


class IteratorReturn:
    def __iter__(self) -> Iterator:
        ...


class IteratorTReturn:
    def __iter__(self) -> Iterator[int]:
        ...


class TypingIteratorReturn:
    def __iter__(self) -> typing.Iterator:
        ...


class TypingIteratorTReturn:
    def __iter__(self) -> typing.Iterator[int]:
        ...


class CollectionsIteratorReturn:
    def __iter__(self) -> collections.abc.Iterator:
        ...


class CollectionsIteratorTReturn:
    def __iter__(self) -> collections.abc.Iterator[int]:
        ...


class TypingAsyncIterableTReturn:
    def __aiter__(self) -> typing.AsyncIterable[int]:
        ...


class TypingAsyncIterableReturn:
    def __aiter__(self) -> typing.AsyncIterable:
        ...



import typing
from typing import TypeVar

_T = typing.TypeVar("_T")
_P = TypeVar("_P")

# OK
_UsedTypeVar = TypeVar("_UsedTypeVar")
def func(arg: _UsedTypeVar) -> _UsedTypeVar: ...

_A, _B = TypeVar("_A"), TypeVar("_B")
_C = _D = TypeVar("_C")



def func():
    try:
       ...
    except Exception:
       ...
    except OtherException as e:
       ...
    else:
       ...
    finally:
       ...

def func():
    try:
       ...
    except Exception:
       ...

def func():
    try:
       ...
    except Exception:
       ...
    except OtherException as e:
       ...

def func():
    try:
       ...
    except Exception:
       ...
    except OtherException as e:
       ...
    else:
       ...

def func():
    try:
       ...
    finally:
       ...



from flask import Flask

app = Flask(__name__)

@app.route('/')
def main():
    raise

# OK
app.run(debug=True)

# Errors
app.run()
app.run(debug=False)

# Unrelated
run()
run(debug=True)
run(debug)
foo.run(debug=True)
app = 1
app.run(debug=True)



import builtins
from abc import abstractmethod


def __repr__(self) -> str:
    ...


def __str__(self) -> builtins.str:
    ...


def __repr__(self, /, foo) -> str:
    ...


def __repr__(self, *, foo) -> str:
    ...


class ShouldRemoveSingle:
    def __str__(self) -> builtins.str:
        ...


class ShouldRemove:
    def __repr__(self) -> str:
        ...

    def __str__(self) -> builtins.str:
        ...


class NoReturnSpecified:
    def __str__(self):
        ...

    def __repr__(self):
        ...


class NonMatchingArgs:
    def __str__(self, *, extra) -> builtins.str:
        ...

    def __repr__(self, /, extra) -> str:
        ...


class MatchingArgsButAbstract:
    @abstractmethod
    def __str__(self) -> builtins.str:
        ...

    @abstractmethod
    def __repr__(self) -> str:
        ...



import os

os.getenv(1)  # [invalid-envvar-value]
os.getenv("a")
os.getenv("test")
os.getenv(key="testingAgain")
os.getenv(key=11)  # [invalid-envvar-value]
os.getenv(["hello"])  # [invalid-envvar-value]
os.getenv(key="foo", default="bar")
os.getenv(key=f"foo", default="bar")
os.getenv(key="foo" + "bar", default=1)
os.getenv(key=1 + "bar", default=1)  # [invalid-envvar-value]
os.getenv("PATH_TEST" if using_clear_path else "PATH_ORIG")
os.getenv(1 if using_clear_path else "PATH_ORIG")

AA = "aa"
os.getenv(AA)



#!/usr/bin/python3

from __future__ import annotations

import argparse
from pathlib import Path


def import_fixture(fixture: Path, fixture_set: str):
    """
    Imports a single fixture by writing the input and expected output to the black directory.
    """

    output_directory = Path(__file__).parent.joinpath("black").joinpath(fixture_set)
    output_directory.mkdir(parents=True, exist_ok=True)

    fixture_path = output_directory.joinpath(fixture.name)
    expect_path = fixture_path.with_suffix(".py.expect")

    with (
        fixture.open("r") as black_file,
        fixture_path.open("w") as fixture_file,
        expect_path.open("w") as expect_file
    ):
        lines = iter(black_file)
        expected = []
        input = []

        for line in lines:
            if line.rstrip() == "# output":
                expected = list(lines)
                break
            else:
                input.append(line)

        if not expected:
            # If there's no output marker, tread the whole file as already pre-formatted
            expected = input

        fixture_file.write("".join(input).strip() + "\n")
        expect_file.write("".join(expected).strip() + "\n")


# The name of the folders in the `data` for which the tests should be imported
FIXTURE_SETS = [
    "fast",
    "py_36",
    "py_37",
    "py_38",
    "py_39",
    "py_310",
    "py_311",
    "py_312",
    "simple_cases",
    "miscellaneous",
    ".",
    "type_comments"
]

# Tests that ruff doesn't fully support yet and, therefore, should not be imported
IGNORE_LIST = [
    # Contain syntax errors
    "async_as_identifier.py",
    "invalid_header.py",
    "pattern_matching_invalid.py",

    # Python 2
    "python2_detection.py"
]


def import_fixtures(black_dir: str):
    """Imports all the black fixtures"""

    test_directory = Path(black_dir, "tests/data")

    if not test_directory.exists():
        print(
            "Black directory does not contain a 'tests/data' directory. Does the directory point to a full black "
            "checkout (git clone https://github.com/psf/black.git)?")
        return

    for fixture_set in FIXTURE_SETS:
        fixture_directory = test_directory.joinpath(fixture_set)
        fixtures = fixture_directory.glob("*.py")

        if not fixtures:
            print(f"Fixture set '{fixture_set}' contains no python files")
            return

        for fixture in fixtures:
            if fixture.name in IGNORE_LIST:
                print(f"Ignoring fixture '{fixture}")
                continue

            print(f"Importing fixture '{fixture}")
            import_fixture(fixture, fixture_set)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="Imports the test suite from black.",
        epilog="import_black_tests.py <path_to_black_repository>"
    )

    parser.add_argument("black_dir", type=Path)

    args = parser.parse_args()

    black_dir = args.black_dir

    import_fixtures(black_dir)



class Bad(tuple):  # SLOT001
    pass


class Good(tuple):  # Ok
    __slots__ = ("foo",)


from typing import Tuple


class Bad(Tuple):  # SLOT001
    pass


class Bad(Tuple[str, int, float]):  # SLOT001
    pass


class Good(Tuple[str, int, float]):  # OK
    __slots__ = ("foo",)



from typing import NoReturn, Never
import typing_extensions


def foo(arg):
    ...


def foo_int(arg: int):
    ...


def foo_no_return(arg: NoReturn):
    ...


def foo_no_return_typing_extensions(
    arg: typing_extensions.NoReturn,
):
    ...


def foo_no_return_kwarg(arg: int, *, arg2: NoReturn):
    ...


def foo_no_return_pos_only(arg: int, /, arg2: NoReturn):
    ...


def foo_never(arg: Never):
    ...



from typing import Annotated, Any, Optional, Type, Union
from typing_extensions import override

# Error
def foo(a, b):
    pass


# Error
def foo(a: int, b):
    pass


# Error
def foo(a: int, b) -> int:
    pass


# Error
def foo(a: int, b: int):
    pass


# Error
def foo():
    pass


# OK
def foo(a: int, b: int) -> int:
    pass


# OK
def foo() -> int:
    pass


# OK
def foo(a: int, *args: str, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: Any, *args: str, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: int, *args: str, **kwargs: str) -> Any:
    pass


# ANN401
def foo(a: int, *args: Any, **kwargs: Any) -> int:
    pass


# ANN401
def foo(a: int, *args: Any, **kwargs: str) -> int:
    pass


# ANN401
def foo(a: int, *args: str, **kwargs: Any) -> int:
    pass


class Foo:
    # OK
    def foo(self: "Foo", a: int, b: int) -> int:
        pass

    # ANN101
    def foo(self, a: int, b: int) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: Any, *params: str, **options: str) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: str, **options: str) -> Any:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: Any, **options: Any) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: Any, **options: str) -> int:
        pass

    # ANN401
    def foo(self: "Foo", a: int, *params: str, **options: Any) -> int:
        pass

    # OK
    @override
    def foo(self: "Foo", a: Any, *params: str, **options: str) -> int:
        pass

    # OK
    @override
    def foo(self: "Foo", a: int, *params: str, **options: str) -> Any:
        pass

    # OK
    @override
    def foo(self: "Foo", a: int, *params: Any, **options: Any) -> int:
        pass

    # OK
    @override
    def foo(self: "Foo", a: int, *params: Any, **options: str) -> int:
        pass

    # OK
    @override
    def foo(self: "Foo", a: int, *params: str, **options: Any) -> int:
        pass

    # OK
    @classmethod
    def foo(cls: Type["Foo"], a: int, b: int) -> int:
        pass

    # ANN102
    @classmethod
    def foo(cls, a: int, b: int) -> int:
        pass

    # ANN101
    def foo(self, /, a: int, b: int) -> int:
        pass


# OK
def f(*args: *tuple[int]) -> None: ...
def f(a: object) -> None: ...
def f(a: str | bytes) -> None: ...
def f(a: Union[str, bytes]) -> None: ...
def f(a: Optional[str]) -> None: ...
def f(a: Annotated[str, ...]) -> None: ...
def f(a: "Union[str, bytes]") -> None: ...
def f(a: int + int) -> None: ...

# ANN401
def f(a: Any | int) -> None: ...
def f(a: int | Any) -> None: ...
def f(a: Union[str, bytes, Any]) -> None: ...
def f(a: Optional[Any]) -> None: ...
def f(a: Annotated[Any, ...]) -> None: ...
def f(a: "Union[str, bytes, Any]") -> None: ...


class Foo:
    @decorator()
    def __init__(self: "Foo", foo: int):
       ...



import os.path
from pathlib import Path
from os.path import getctime


os.path.getctime("filename")
os.path.getctime(b"filename")
os.path.getctime(Path("filename"))

getctime("filename")
getctime(b"filename")
getctime(Path("filename"))



# ok
with open("/abc/tmp", "w") as f:
    f.write("def")

with open("/tmp/abc", "w") as f:
    f.write("def")

with open("/var/tmp/123", "w") as f:
    f.write("def")

with open("/dev/shm/unit/test", "w") as f:
    f.write("def")

# not ok by config
with open("/foo/bar", "w") as f:
    f.write("def")

# Using `tempfile` module should be ok
import tempfile
from tempfile import TemporaryDirectory

with tempfile.NamedTemporaryFile(dir="/tmp") as f:
    f.write(b"def")

with tempfile.NamedTemporaryFile(dir="/var/tmp") as f:
    f.write(b"def")

with tempfile.TemporaryDirectory(dir="/dev/shm") as d:
    pass

with TemporaryDirectory(dir="/tmp") as d:
    pass



"""Test case: `Literal` with `__future__` annotations."""

from __future__ import annotations

from typing import Literal, Final

from typing_extensions import assert_type

CONSTANT: Final = "ns"

assert_type(CONSTANT, Literal["ns"])



import sys


def print_python_version():
    print(sys.version)
    return None  # [useless-return]


def print_python_version():
    print(sys.version)
    return None  # [useless-return]


def print_python_version():
    print(sys.version)
    return None  # [useless-return]


class SomeClass:
    def print_python_version(self):
        print(sys.version)
        return None  # [useless-return]


def print_python_version():
    if 2 * 2 == 4:
        return
    print(sys.version)


def print_python_version():
    if 2 * 2 == 4:
        return None
    return


def print_python_version():
    if 2 * 2 == 4:
        return None


def print_python_version():
    """This function returns None."""
    return None


def print_python_version():
    """This function returns None."""
    print(sys.version)
    return None  # [useless-return]


class BaseCache:
    def get(self, key: str) -> str | None:
        print(f"{key} not found")
        return None

    def get(self, key: str) -> None:
        print(f"{key} not found")
        return None



def f():
    from typing import Literal

    # OK
    x: Literal["foo"]


def f():
    from .typical import Literal

    # OK
    x: Literal["foo"]


def f():
    from . import typical

    # OK
    x: typical.Literal["foo"]


def f():
    from .atypical import Literal

    # F821
    x: Literal["foo"]



from typing import TypeVar, Self, Type

_S = TypeVar("_S", bound=BadClass)
_S2 = TypeVar("_S2", BadClass, GoodClass)

class BadClass:
    def __new__(cls: type[_S], *args: str, **kwargs: int) -> _S: ...  # PYI019


    def bad_instance_method(self: _S, arg: bytes) -> _S: ...  # PYI019


    @classmethod
    def bad_class_method(cls: type[_S], arg: int) -> _S: ...  # PYI019


    @classmethod
    def bad_posonly_class_method(cls: type[_S], /) -> _S: ...  # PYI019


    @classmethod
    def excluded_edge_case(cls: Type[_S], arg: int) -> _S: ...  # Ok


class GoodClass:
    def __new__(cls: type[Self], *args: list[int], **kwargs: set[str]) -> Self: ...
    def good_instance_method_1(self: Self, arg: bytes) -> Self: ...
    def good_instance_method_2(self, arg1: _S2, arg2: _S2) -> _S2: ...
    @classmethod
    def good_cls_method_1(cls: type[Self], arg: int) -> Self: ...
    @classmethod
    def good_cls_method_2(cls, arg1: _S, arg2: _S) -> _S: ...
    @staticmethod
    def static_method(arg1: _S) -> _S: ...


# Python > 3.12
class PEP695BadDunderNew[T]:
  def __new__[S](cls: type[S], *args: Any, ** kwargs: Any) -> S: ...  # PYI019


  def generic_instance_method[S](self: S) -> S: ...  # PYI019


class PEP695GoodDunderNew[T]:
  def __new__(cls, *args: Any, **kwargs: Any) -> Self: ...



from collections import namedtuple
from typing import NamedTuple


class Bad(namedtuple("foo", ["str", "int"])):  # SLOT002
    pass


class Good(namedtuple("foo", ["str", "int"])):  # OK
    __slots__ = ("foo",)


class Good(NamedTuple):  # Ok
    pass



from __future__ import annotations


def f_a():
    raise RuntimeError("This is an example exception")


def f_a_short():
    raise RuntimeError("Error")


def f_a_empty():
    raise RuntimeError("")


def f_b():
    example = "example"
    raise RuntimeError(f"This is an {example} exception")


def f_c():
    raise RuntimeError("This is an {example} exception".format(example="example"))


def f_ok():
    msg = "hello"
    raise RuntimeError(msg)


def f_unfixable():
    msg = "hello"
    raise RuntimeError("This is an example exception")


def f_msg_in_nested_scope():
    def nested():
        msg = "hello"

    raise RuntimeError("This is an example exception")


def f_msg_in_parent_scope():
    msg = "hello"

    def nested():
        raise RuntimeError("This is an example exception")


def f_fix_indentation_check(foo):
    if foo:
        raise RuntimeError("This is an example exception")
    else:
        if foo == "foo":
            raise RuntimeError(f"This is an exception: {foo}")
    raise RuntimeError("This is an exception: {}".format(foo))


# Report these, but don't fix them
if foo: raise RuntimeError("This is an example exception")
if foo: x = 1; raise RuntimeError("This is an example exception")



"""Vendored from scripts/mkstdlibs.py in PyCQA/isort.

Source:
    https://github.com/PyCQA/isort/blob/e321a670d0fefdea0e04ed9d8d696434cf49bdec/scripts/mkstdlibs.py

Only the generation of the file has been modified for use in this project.
"""
from __future__ import annotations

from pathlib import Path

from sphinx.ext.intersphinx import fetch_inventory

URL = "https://docs.python.org/{}/objects.inv"
PATH = Path("crates") / "ruff_python_stdlib" / "src" / "sys.rs"
VERSIONS: list[tuple[int, int]] = [
    (3, 7),
    (3, 8),
    (3, 9),
    (3, 10),
    (3, 11),
    (3, 12),
]


class FakeConfig:
    intersphinx_timeout = None
    tls_verify = True
    user_agent = ""


class FakeApp:
    srcdir = ""
    config = FakeConfig()


with PATH.open("w") as f:
    f.write(
        """\
//! This file is generated by `scripts/generate_known_standard_library.py`

pub fn is_known_standard_library(minor_version: u32, module: &str) -> bool {
    matches!((minor_version, module),
""",
    )

    modules_by_version = {}

    for major_version, minor_version in VERSIONS:
        url = URL.format(f"{major_version}.{minor_version}")
        invdata = fetch_inventory(FakeApp(), "", url)

        modules = {
            "_ast",
            "posixpath",
            "ntpath",
            "sre_constants",
            "sre_parse",
            "sre_compile",
            "sre",
        }

        for module in invdata["py:module"]:
            root, *_ = module.split(".")
            if root not in ["__future__", "__main__"]:
                modules.add(root)

        modules_by_version[minor_version] = modules

    # First, add a case for the modules that are in all versions.
    ubiquitous_modules = set.intersection(*modules_by_version.values())

    f.write("(_, ")
    for i, module in enumerate(sorted(ubiquitous_modules)):
        if i > 0:
            f.write(" | ")
        f.write(f'"{module}"')
    f.write(")")
    f.write("\n")

    # Next, add any version-specific modules.
    for _major_version, minor_version in VERSIONS:
        version_modules = set.difference(
            modules_by_version[minor_version],
            ubiquitous_modules,
        )

        f.write(" | ")
        f.write(f"({minor_version}, ")
        for i, module in enumerate(sorted(version_modules)):
            if i > 0:
                f.write(" | ")
            f.write(f'"{module}"')
        f.write(")")
        f.write("\n")

    f.write(
        """\
        )
}
    """,
    )



# SIM201
if not a == b:
    pass

# SIM201
if not a == (b + c):
    pass

# SIM201
if not (a + b) == c:
    pass

# OK
if not a != b:
    pass

# OK
if a == b:
    pass

# OK
if not a == b:
    raise ValueError()

# OK
def __ne__(self, other):
    return not self == other



def a():
    return

def __init__():
    return

class A:
    def __init__(self):
        return


class B:
    def __init__(self):
        return 3

    def gen(self):
        return 5

class MyClass:

    def __init__(self):
        return 1

class MyClass2:
    """dummy class"""

    def __init__(self):
        return


class MyClass3:
    """dummy class"""

    def __init__(self):
        return None

class MyClass5:
    """dummy class"""

    def __init__(self):
        self.callable = lambda: (yield None)



from django.db import models
from django.db.models import Model


class StrBeforeRandomField(models.Model):
    """Model with `__str__` before a random property."""

    class Meta:
        verbose_name = "test"
        verbose_name_plural = "tests"

    def __str__(self):
        return ""

    random_property = "foo"


class StrBeforeFieldModel(models.Model):
    """Model with `__str__` before fields."""

    class Meta:
        verbose_name = "test"
        verbose_name_plural = "tests"

    def __str__(self):
        return "foobar"

    first_name = models.CharField(max_length=32)


class ManagerBeforeField(models.Model):
    """Model with manager before fields."""

    objects = "manager"

    class Meta:
        verbose_name = "test"
        verbose_name_plural = "tests"

    def __str__(self):
        return "foobar"

    first_name = models.CharField(max_length=32)


class CustomMethodBeforeStr(models.Model):
    """Model with a custom method before `__str__`."""

    class Meta:
        verbose_name = "test"
        verbose_name_plural = "tests"

    def my_method(self):
        pass

    def __str__(self):
        return "foobar"


class GetAbsoluteUrlBeforeSave(Model):
    """Model with `get_absolute_url` method before `save` method.

    Subclass this directly using the `Model` class.
    """

    def get_absolute_url(self):
        pass

    def save(self):
        pass


class ConstantsAreNotFields(models.Model):
    """Model with an assignment to a constant after `__str__`."""

    first_name = models.CharField(max_length=32)

    class Meta:
        verbose_name = "test"
        verbose_name_plural = "tests"

    def __str__(self):
        pass

    MY_CONSTANT = id(1)


class PerfectlyFine(models.Model):
    """Model which has everything in perfect order."""

    first_name = models.CharField(max_length=32)
    last_name = models.CharField(max_length=32)
    objects = "manager"

    class Meta:
        verbose_name = "test"
        verbose_name_plural = "tests"

    def __str__(self):
        return "Perfectly fine!"

    def save(self, **kwargs):
        super(PerfectlyFine, self).save(**kwargs)

    def get_absolute_url(self):
        return "http://%s" % self

    def my_method(self):
        pass

    @property
    def random_property(self):
        return "%s" % self


class MultipleConsecutiveFields(models.Model):
    """Model that contains multiple out-of-order field definitions in a row."""


    class Meta:
        verbose_name = "test"

    first_name = models.CharField(max_length=32)
    last_name = models.CharField(max_length=32)

    def get_absolute_url(self):
        pass

    middle_name = models.CharField(max_length=32)



def f():
    # This should ignore both errors.
    from typing import (  # noqa: F401
        List,
        Sequence,
    )


def f():
    # This should ignore both errors.
    from typing import (  # noqa
        List,
        Sequence,
    )


def f():
    # This should ignore both errors.
    from typing import (
        List,  # noqa: F401
        Sequence,  # noqa: F401
    )


def f():
    # This should ignore both errors.
    from typing import (
        List,  # noqa
        Sequence,  # noqa
    )


def f():
    # This should ignore the first error.
    from typing import (
        Mapping,  # noqa: F401
        Union,
    )


def f():
    # This should ignore both errors.
    from typing import (  # noqa
        List,
        Sequence,
    )


def f():
    # This should ignore the error, but the inner noqa should be marked as unused.
    from typing import (  # noqa: F401
        Optional,  # noqa: F401
    )


def f():
    # This should ignore the error, but the inner noqa should be marked as unused.
    from typing import (  # noqa
        Optional,  # noqa: F401
    )


def f():
    # This should ignore the error, but mark F501 as unused.
    from typing import (  # noqa: F401
        Dict,  # noqa: F501
    )


def f():
    # This should ignore the error, but mark F501 as unused.
    from typing import (  # noqa: F501
        Tuple,  # noqa: F401
    )


def f():
    # This should ignore both errors.
    from typing import Any, AnyStr  # noqa: F401


def f():
    # This should ignore both errors.
    from typing import AsyncIterable, AsyncGenerator  # noqa


def f():
    # This should mark F501 as unused.
    from typing import Awaitable, AwaitableGenerator  # noqa: F501



from __future__ import annotations

import re
from pathlib import Path

ROOT_DIR = Path(__file__).resolve().parent.parent


def dir_name(linter_name: str) -> str:
    return linter_name.replace("-", "_")


def pascal_case(linter_name: str) -> str:
    """Convert from snake-case to PascalCase."""
    return "".join(word.title() for word in linter_name.split("-"))


def snake_case(name: str) -> str:
    """Convert from PascalCase to snake_case."""
    return "".join(
        f"_{word.lower()}" if word.isupper() else word for word in name
    ).lstrip("_")


def get_indent(line: str) -> str:
    return re.match(r"^\s*", line).group()  # type: ignore[union-attr]



"""
Should emit:
B032 - on lines 9, 10, 12, 13, 16-19
"""

# Flag these
dct = {"a": 1}

dct["b"]: 2
dct.b: 2

dct["b"]: "test"
dct.b: "test"

test = "test"
dct["b"]: test
dct["b"]: test.lower()
dct.b: test
dct.b: test.lower()

# Do not flag below
typed_dct: dict[str, int] = {"a": 1}
typed_dct["b"] = 2
typed_dct.b = 2


class TestClass:
    def test_self(self):
        self.test: int


"""
Violation:

Returning a final value inside a try block may indicate you could use an else block
instead to outline the success scenario
"""
import logging

logger = logging.getLogger(__name__)


class MyException(Exception):
    pass


def bad():
    try:
        a = 1
        b = process()
        return b
    except MyException:
        logger.exception("process failed")


def good():
    try:
        a = 1
        b = process()
    except MyException:
        logger.exception("process failed")
    else:
        return b


def noreturn():
    try:
        a = 1
        b = process()
    except MyException:
        logger.exception("process failed")


def good_return_with_side_effects():
    try:
        pass
        return process()
    except MyException:
        logger.exception("process failed")

def good_noexcept():
    try:
        pass
        return process()
    finally:
        logger.exception("process failed")



def bad_function():
    """this docstring is not capitalized"""

def good_function():
    """This docstring is capitalized."""

def other_function():
    """
    This docstring is capitalized."""

def another_function():
    """ This docstring is capitalized."""

def utf8_function():
    """éste docstring is capitalized."""

def uppercase_char_not_possible():
    """'args' is not capitalized."""

def non_alphabetic():
    """th!is is not capitalized."""

def non_ascii():
    """th•s is not capitalized."""

def all_caps():
    """th•s is not capitalized."""



import sys

if sys.platform == "platform_name_1": ...  # OK

if sys.platform != "platform_name_2": ...  # OK

if sys.platform in ["linux"]: ...  # Error: PYI007 Unrecognized sys.platform check

if sys.platform > 3: ...  # Error: PYI007 Unrecognized sys.platform check

if sys.platform == 10.12: ...  # Error: PYI007 Unrecognized sys.platform check



"""Regression test: ensure that we don't treat the export entry as a typing-only reference."""
from __future__ import annotations

from logging import getLogger

__all__ = ("getLogger",)


def foo() -> None:
    pass



"""
Should emit:
B002 - on lines 18, 19, and 24
"""


def this_is_all_fine(n):
    x = n + 1
    y = 1 + n
    z = +x + y
    a = n - 1
    b = 1 - n
    c = -a - b
    return +z, -c


def this_is_buggy(n):
    x = ++n
    y = --n
    return x, y


def this_is_buggy_too(n):
    return ++n, --n



def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        if i % 2:
            result.append(i)  # PERF401


def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        result.append(i * i)  # PERF401


def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        if i % 2:
            result.append(i)  # Ok
        elif i % 2:
            result.append(i)
        else:
            result.append(i)


def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        result.append(i)  # OK


def f():
    items = [1, 2, 3, 4]
    result = {}
    for i in items:
        result[i].append(i)  # OK


def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        if i not in result:
            result.append(i)  # OK


def f():
    fibonacci = [0, 1]
    for i in range(20):
        fibonacci.append(sum(fibonacci[-2:]))  # OK
    print(fibonacci)


def f():
    foo = object()
    foo.fibonacci = [0, 1]
    for i in range(20):
        foo.fibonacci.append(sum(foo.fibonacci[-2:]))  # OK
    print(foo.fibonacci)


class Foo:
    def append(self, x):
        pass


def f():
    items = [1, 2, 3, 4]
    result = Foo()
    for i in items:
        result.append(i)  # Ok



dict((x, x) for x in range(3))
dict(
    (x, x) for x in range(3)
)
dict(((x, x) for x in range(3)), z=3)
y = f'{dict((x, x) for x in range(3))}'
print(f'Hello {dict((x, x) for x in range(3))} World')
print(f"Hello {dict((x, x) for x in 'abc')} World")
print(f'Hello {dict((x, x) for x in "abc")} World')
print(f'Hello {dict((x,x) for x in "abc")} World')

f'{dict((x, x) for x in range(3)) | dict((x, x) for x in range(3))}'
f'{ dict((x, x) for x in range(3)) | dict((x, x) for x in range(3)) }'

def f(x):
    return x

print(f'Hello {dict((x,f(x)) for x in "abc")} World')

# Regression test for: https://github.com/astral-sh/ruff/issues/7086
dict((k,v)for k,v in d.iteritems() if k in only_args)

# Regression test for: https://github.com/astral-sh/ruff/issues/7455#issuecomment-1722458940
dict((*v, k) for k, v in enumerate(calendar.month_abbr))



import subprocess
from subprocess import run

# Errors
subprocess.run(["foo"], universal_newlines=True, check=True)
subprocess.run(["foo"], universal_newlines=True, text=True)
run(["foo"], universal_newlines=True, check=False)

# OK
subprocess.run(["foo"], check=True)



# PERF203
for i in range(10):
    try:
        print(f"{i}")
    except:
        print("error")

# OK
try:
    for i in range(10):
        print(f"{i}")
except:
    print("error")

# OK
i = 0
while i < 10:
    try:
        print(f"{i}")
    except:
        print("error")

    i += 1

# OK - no other way to write this
for i in range(10):
    try:
        print(f"{i}")
        break
    except:
        print("error")

# OK - no other way to write this
for i in range(10):
    try:
        print(f"{i}")
        continue
    except:
        print("error")


# OK - no other way to write this
for i in range(10):
    try:
        print(f"{i}")
        if i > 0:
            break
    except:
        print("error")



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Final, Literal, TypeAlias

    RatingKey: TypeAlias = Literal["good", "fair", "poor"]

RATING_KEYS: Final[tuple[RatingKey, ...]] = ("good", "fair", "poor")



import typing
from typing import TypedDict


class _UnusedTypedDict(TypedDict):
    foo: str


class _UnusedTypedDict2(typing.TypedDict):
    bar: int


class _UsedTypedDict(TypedDict):
   foo: bytes


class _CustomClass(_UsedTypedDict):
    bar: list[int]



x = "a string"
y = "another string"
z = ""


def errors():
    if x is "" or x == "":
        print("x is an empty string")

    if y is not "" or y != "":
        print("y is not an empty string")

    if "" != z:
        print("z is an empty string")


def ok():
    if x and not y:
        print("x is not an empty string, but y is an empty string")


data.loc[data["a"] != ""]
data.loc[data["a"] != "", :]



# SIM117
with A() as a:
    with B() as b:
        print("hello")

# SIM117
with A():
    with B():
        with C():
            print("hello")

# SIM117
with A() as a:
    # Unfixable due to placement of this comment.
    with B() as b:
        print("hello")

# SIM117
with A() as a:
    with B() as b:
        # Fixable due to placement of this comment.
        print("hello")

# OK
with A() as a:
    a()
    with B() as b:
        print("hello")

# OK
with A() as a:
    with B() as b:
        print("hello")
    a()

# OK, can't merge async with and with.
async with A() as a:
    with B() as b:
        print("hello")

# OK, can't merge async with and with.
with A() as a:
    async with B() as b:
        print("hello")

# SIM117
async with A() as a:
    async with B() as b:
        print("hello")

while True:
    # SIM117
    with A() as a:
        with B() as b:
            """this
is valid"""

            """the indentation on
            this line is significant"""

            "this is" \
"allowed too"

            ("so is"
"this for some reason")

# SIM117
with (
    A() as a,
    B() as b,
):
    with C() as c:
        print("hello")

# SIM117
with A() as a:
    with (
        B() as b,
        C() as c,
    ):
        print("hello")

# SIM117
with (
    A() as a,
    B() as b,
):
    with (
        C() as c,
        D() as d,
    ):
        print("hello")

# SIM117 (auto-fixable)
with A("01ß9💣2ℝ8901ß9💣2ℝ8901ß9💣2ℝ89") as a:
    with B("01ß9💣2ℝ8901ß9💣2ℝ8901ß9💣2ℝ89") as b:
        print("hello")

# SIM117 (not auto-fixable too long)
with A("01ß9💣2ℝ8901ß9💣2ℝ8901ß9💣2ℝ890") as a:
    with B("01ß9💣2ℝ8901ß9💣2ℝ8901ß9💣2ℝ89") as b:
        print("hello")

# From issue #3025.
async def main():
    async with A() as a: # SIM117.
        async with B() as b:
            print("async-inside!")

    return 0

# OK. Can't merge across different kinds of with statements.
with a as a2:
    async with b as b2:
        with c as c2:
            async with d as d2:
                f(a2, b2, c2, d2)

# OK. Can't merge across different kinds of with statements.
async with b as b2:
    with c as c2:
        async with d as d2:
            f(b2, c2, d2)



import mod.BadAllowed as BADALLOWED
import mod.StillBad as STILLBAD

from mod import BadAllowed as BADALLOWED
from mod import StillBad as STILLBAD

from mod import BadAllowed as BAD_ALLOWED
from mod import StillBad as STILL_BAD



def f():
    # SIM103
    if a:
        return True
    else:
        return False


def f():
    # SIM103
    if a == b:
        return True
    else:
        return False


def f():
    # SIM103
    if a:
        return 1
    elif b:
        return True
    else:
        return False


def f():
    # SIM103
    if a:
        return 1
    else:
        if b:
            return True
        else:
            return False


def f():
    # OK
    if a:
        foo()
        return True
    else:
        return False


def f():
    # OK
    if a:
        return "foo"
    else:
        return False


def f():
    # SIM103 (but not fixable)
    if a:
        return False
    else:
        return True


def f():
    # OK
    if a:
        return False
    else:
        return False


def f():
    # OK
    if a:
        return True
    else:
        return True


def f():
    # OK
    def bool():
        return False
    if a:
        return True
    else:
        return False



books = ["Dune", "Foundation", "Neuromancer"]

# Errors
for index, _ in enumerate(books):
    print(index)

for index, _ in enumerate(books, start=0):
    print(index)

for index, _ in enumerate(books, 0):
    print(index)

for index, _ in enumerate(books, start=1):
    print(index)

for index, _ in enumerate(books, 1):
    print(index)

for index, _ in enumerate(books, start=x):
    print(book)

for index, _ in enumerate(books, x):
    print(book)

for _, book in enumerate(books):
    print(book)

for _, book in enumerate(books, start=0):
    print(book)

for _, book in enumerate(books, 0):
    print(book)

for _, book in enumerate(books, start=1):
    print(book)

for _, book in enumerate(books, 1):
    print(book)

for _, book in enumerate(books, start=x):
    print(book)

for _, book in enumerate(books, x):
    print(book)

for index, (_, _) in enumerate(books):
    print(index)

for (_, _), book in enumerate(books):
    print(book)

for(index, _)in enumerate(books):
    print(index)

for(index), _ in enumerate(books):
    print(index)

# OK
for index, book in enumerate(books):
    print(index, book)

for index in range(len(books)):
    print(index)

for book in books:
    print(book)



import typing

# Shouldn't affect non-union field types.
field1: str

# Should emit for duplicate field types.
field2: str | str  # PYI016: Duplicate union member `str`

# Should emit for union types in arguments.
def func1(arg1: int | int):  # PYI016: Duplicate union member `int`
    print(arg1)

# Should emit for unions in return types.
def func2() -> str | str:  # PYI016: Duplicate union member `str`
    return "my string"

# Should emit in longer unions, even if not directly adjacent.
field3: str | str | int  # PYI016: Duplicate union member `str`
field4: int | int | str  # PYI016: Duplicate union member `int`
field5: str | int | str  # PYI016: Duplicate union member `str`
field6: int | bool | str | int  # PYI016: Duplicate union member `int`

# Shouldn't emit for non-type unions.
field7 = str | str

# Should emit for strangely-bracketed unions.
field8: int | (str | int)  # PYI016: Duplicate union member `int`

# Should handle user brackets when fixing.
field9: int | (int | str)  # PYI016: Duplicate union member `int`
field10: (str | int) | str  # PYI016: Duplicate union member `str`

# Should emit for nested unions.
field11: dict[int | int, str]

# Should emit for unions with more than two cases
field12: int | int | int  # Error
field13: int | int | int | int  # Error

# Should emit for unions with more than two cases, even if not directly adjacent
field14: int | int | str | int  # Error

# Should emit for duplicate literal types; also covered by PYI030
field15: typing.Literal[1] | typing.Literal[1]  # Error

# Shouldn't emit if in new parent type
field16: int | dict[int, str]  # OK

# Shouldn't emit if not in a union parent
field17: dict[int, int]  # OK

# Should emit in cases with newlines
field18: typing.Union[
    set[
        int  # foo
    ],
    set[
        int  # bar
    ],
]  # Error, newline and comment will not be emitted in message

# Should emit in cases with `typing.Union` instead of `|`
field19: typing.Union[int, int]  # Error

# Should emit in cases with nested `typing.Union`
field20: typing.Union[int, typing.Union[int, str]]  # Error

# Should emit in cases with mixed `typing.Union` and `|`
field21: typing.Union[int, int | str]  # Error

# Should emit only once in cases with multiple nested `typing.Union`
field22: typing.Union[int, typing.Union[int, typing.Union[int, int]]]  # Error

# Should emit in cases with newlines
field23: set[  # foo
    int] | set[int]

# Should emit twice (once for each `int` in the nested union, both of which are
# duplicates of the outer `int`), but not three times (which would indicate that
# we incorrectly re-checked the nested union).
field24: typing.Union[int, typing.Union[int, int]]  # PYI016: Duplicate union member `int`

# Should emit twice (once for each `int` in the nested union, both of which are
# duplicates of the outer `int`), but not three times (which would indicate that
# we incorrectly re-checked the nested union).
field25: typing.Union[int, int | int]  # PYI016: Duplicate union member `int`



import logging

logger = logging.getLogger(__name__)

logging.exception("foo")  # OK
logging.exception("foo", exc_info=False)  # LOG007
logging.exception("foo", exc_info=[])  # LOG007
logger.exception("foo")  # OK
logger.exception("foo", exc_info=False)  # LOG007
logger.exception("foo", exc_info=[])  # LOG007
logger.error("foo", exc_info=False)  # OK
logger.info("foo", exc_info=False)  # OK


from logging import exception

exception("foo", exc_info=False)  # LOG007
exception("foo", exc_info=True)  # OK
